# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class NEMImportanceTransfer(p.MessageType):
    FIELDS = {
        1: ('mode', p.UVarintType, 0),
        2: ('public_key', p.BytesType, 0),
    }

    def __init__(
        self,
        mode: int = None,
        public_key: bytes = None,
    ) -> None:
        self.mode = mode
        self.public_key = public_key
