# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p
from .MoneroKeyImageExportInitRequest import MoneroKeyImageExportInitRequest
from .MoneroKeyImageSyncFinalRequest import MoneroKeyImageSyncFinalRequest
from .MoneroKeyImageSyncStepRequest import MoneroKeyImageSyncStepRequest


class MoneroKeyImageSyncRequest(p.MessageType):
    MESSAGE_WIRE_TYPE = 511
    FIELDS = {
        1: ('init', MoneroKeyImageExportInitRequest, 0),
        2: ('step', MoneroKeyImageSyncStepRequest, 0),
        3: ('final_msg', MoneroKeyImageSyncFinalRequest, 0),
    }

    def __init__(
        self,
        init: MoneroKeyImageExportInitRequest = None,
        step: MoneroKeyImageSyncStepRequest = None,
        final_msg: MoneroKeyImageSyncFinalRequest = None,
    ) -> None:
        self.init = init
        self.step = step
        self.final_msg = final_msg
