#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Author: Dusan Klinec, ph4r05, 2018


def collect(*args, **kwargs):
    pass


def mem_free(*args, **kwargs):
    return 1000


def mem_alloc(*args, **kwargs):
    return 100
