# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class MoneroKeyImageSyncFinalAck(p.MessageType):
    MESSAGE_WIRE_TYPE = 522
    FIELDS = {
        1: ('enc_key', p.BytesType, 0),
    }

    def __init__(
        self,
        enc_key: bytes = None,
    ) -> None:
        self.enc_key = enc_key
