# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class CardanoTxAck(p.MessageType):
    MESSAGE_WIRE_TYPE = 309
    FIELDS = {
        1: ('transaction', p.BytesType, 0),
    }

    def __init__(
        self,
        transaction: bytes = None,
    ) -> None:
        self.transaction = transaction
